;; -*- no-byte-compile: t; lexical-binding: nil -*-
(define-package "python-black" "1.3.0"
  "Reformat Python using python-black."
  '((emacs       "25")
    (dash        "2.16.0")
    (reformatter "0.3"))
  :url "https://github.com/wbolster/emacs-python-black"
  :commit "779d49c7db54590d1fa483ef2f89eea5ef8774e1"
  :revdesc "1.3.0-0-g779d49c7db54"
  :keywords '("languages")
  :authors '(("wouter bolsterlee" . "wouter@bolsterl.ee"))
  :maintainers '(("wouter bolsterlee" . "wouter@bolsterl.ee")))
